import os, time
from tqdm import tqdm
import itertools
from pathlib import Path

from collections import namedtuple

import numpy as np
import pandas as pd

from tsl import *
from config import *

class DLFinancial:

    def __init__(self):

        self.table_id_to_table_name = {
            18 : '分红送股',
            40 : '业绩预测',
            41 : '业绩快报',
            42 : '主要财务指标',
            44 : '合并资产负债表',
            46 : '合并利润分配表',
            48 : '合并现金流量表',
            56 : '金融公司资产负债表',
            58 : '金融公司利润分配表',
            60 : '金融公司现金流量表',
        }

        self.table_id_to_index_list = {
            18 : ['StockID', 'StockName', '截止日'],
            40 : ['StockID', 'StockName', '截止日', '公布日'],
            41 : ['StockID', 'StockName', '截止日', '公布日'],
            42 : ['StockID', 'StockName', '截止日', '公布日'],
            44 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
            46 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
            48 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
            56 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
            58 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
            60 : ['StockID', 'StockName', '截止日', '数据报告期', '公布日'],
        }

        self.config_name_list = [
            '每股指标',
            '盈利能力',
            '偿债能力',
            '资本结构',
            '经营能力',
            '投资收益',
            '成长能力',
            '现金流指标',
            '资产负债表结构',
            '利润分配表结构',
            '现金流量表结构',
            '估值指标',
        ]


    def do_fin_report(self):
        for table_id in self.table_id_to_table_name.keys():
            self._dump_fin_report(table_id, 20000101)


    def _dump_fin_report(self, table_id, report_start_date):
        
        table_name = self.table_id_to_table_name[table_id]
        dump_folder = '{}/基础报表/{}.{}/'.format(TINYSOFT_DATA_PATH, table_id, table_name)
        Path(dump_folder).mkdir(parents=True, exist_ok=True)
        
        def _dump_df_to_csv(dump_folder, stock_id, df):
            dump_path = '{}/{}.csv'.format(dump_folder, stock_id)

            index_cols = self.table_id_to_index_list[table_id]
            df.set_index(index_cols, inplace=True)
            df.to_csv(dump_path)  

        with tsl() as ts:
            stock_list = ts.get_stock_list()
            print('正在获取数据：', table_id, table_name)

            with tqdm(stock_list) as pbar:
                for stock_id in pbar:
                    pbar.set_description(dump_folder + stock_id)

                    df = ts.get_cmp_report(
                        table_id=table_id, 
                        stock_id=stock_id, 
                        start_date=report_start_date
                    )

                    if df.shape[0] == 0 or df.shape[1] == 0:
                        print('{}的{}为空数据'.format(stock_id, table_name))
                        continue

                    _dump_df_to_csv(dump_folder, stock_id, df)
                    time.sleep(0.001)


    def _dump_common_info(self):
        pass


    def _dump_derived_indicators(self, report_start_date):
                
        def _dump_df_to_csv(config_name, stock_id, df):
            dump_path = '{}/衍生指标/{}/{}.csv'.format(
                TINYSOFT_DATA_PATH, config_name, stock_id)
            df.to_csv(dump_path)

        with tsl() as ts:
            stock_list = ts.get_stock_list()

            for config_name in config_name_list:
                print('正在获取数据：', config_name)
                
                with tqdm(stock_list) as pbar:
                    for stock_id in pbar:
                        pbar.set_description(stock_id)
                        
                        df = ts.get_cmp_indicator(
                            stock_id=stock_id, 
                            start_year=report_start_year, 
                            indicator_config_fname='{}/indicator-config/{}.csv'.format(
                                TINYSOFT_DATA_PATH,
                                config_name
                            )
                        )
                        _dump_df_to_csv(config_name, stock_id, df)
                        time.sleep(0.001)


if __name__ == '__main__':
    DLFinancial().do_fin_report()

