# -*- coding: utf-8 -*-
from BatchImports.Common import *;
import Data.DataFeed.SqlCommonFuncs as sqlCF;
from .Connect import *;
import TSLPy3 as ts;


# 从天软获取指定板块在指定日期的所有股票
# returnType支持：int和str，str为天软格式
# connectionSafe: 为True时，会在结束时断开连接
def getBK(bkName, dateInt: int, returnType=str, connectionSafe=True):
    login();

    tsCode = """
        SetSysParam(Pn_Bk(),"%s");
        EndT:=inttodate(%d);
        return Bk_Listed(EndT);
    """ % (bkName, dateInt);

    data = ts.RemoteExecute(tsCode, {});
    if data[0] == 0:
        if returnType == str:
            stockLi = [str(s, encoding='utf-8') for s in data[1]];
        elif returnType == int:
            stockLi = [int(str(s, encoding='utf-8')[2:]) for s in data[1]];
        else:
            raise Exception("getBK暂时不支持返回此格式");
    else:
        stockLi = None;

    if connectionSafe:
        logoff();

    return stockLi;

# 从天软获取指定日期的上市所有股票
# returnType支持：int和str，str为天软格式
# connectionSafe: 为True时，会在结束时断开连接
def getListedStock(dateInt: int, returnType=str, connectionSafe=True):
    return getBK('A股', dateInt, returnType, connectionSafe);

# 从天软获取交易日信息
# returnType支持：int和datetime
# connectionSafe: 为True时，会在结束时断开连接
def getTradingDays(sDateInt: int, eDateInt: int, returnType=int, connectionSafe=True):
    login();

    tsCode = """
       begt:=inttodate(%d);
       endt:=inttodate(%d);
       return spec(specdate(nday3(tradedays(begt,endt),DateToInt(sp_time())),endt),'SH000001');
    """ % (sDateInt, eDateInt);

    data = ts.RemoteExecute(tsCode, {});
    if data[0] == 0:
        if returnType == int:
            dateLi = data[1];
        elif returnType == datetime:
            dateLi = [datetime.strptime(str(s), "%Y%m%d") for s in data[1]];
        else:
            raise Exception("getTradingDays暂时不支持返回此格式");
    else:
        dateLi = None;

    if connectionSafe:
        logoff();

    return dateLi;

# 获取指定日期的股票日K线
def getStockDailyKLines(dateInt, stockLi=None, connectionSafe = True):
    login();
    if stockLi is None:
        stockLi = getListedStock(dateInt, returnType=str, connectionSafe=False);

    stockCodeArr = sqlCF.mergeStr(stockLi);


    ## 获取基本的K线字段
    tsCode = """
        setsysparam(pn_cycle(),cy_day());
        a:=array(%s);
        t:=select ['StockID'],datetoint(['date']) as 'date',['open'],['high'],['low'],['close'], ['vol'], ['amount'], ['cjbs'],['yclose']
            from markettable datekey %dT to %dT+0.999 of a end;
        return t;
    """ % (stockCodeArr, dateInt, dateInt);

    # 天软去除的数据格式为list[dict[field, value]]，fieldbyte形式的，其他是数值。
    data = ts.RemoteExecute(tsCode, {});


    # data[0] == 0 表示返回没有错误。
    if data[0] == 0:
        stocks = [str(s[b'StockID'], encoding='utf-8') for s in data[1]];
        openLi = [s[b'open'] for s in data[1]];
        highLi = [s[b'high'] for s in data[1]];
        lowLi = [s[b'low'] for s in data[1]];
        closeLi = [s[b'close'] for s in data[1]];
        volLi = [s[b'vol'] for s in data[1]];
        amtLi = [s[b'amount'] for s in data[1]];
        preClsLi = [s[b'yclose'] for s in data[1]];
        tradeNumLi = [s[b'cjbs'] for s in data[1]];

        klineDf = DataFrame({"Date": datetime.strptime(str(dateInt), '%Y%m%d'), "Open":openLi, "High":highLi, "Low":lowLi, "Close":closeLi, "Volume":volLi,
                             "Amount":amtLi, "PreClose":preClsLi, "TradeNum":tradeNumLi}, index = stocks);
    else:
        klineDf = None;



    ## 补充涨跌幅和ST字段
    ## 获取基本的K线字段
    tsCode2 = """
        endT := %d;
        setsysparam(pn_cycle(),cy_day());
        SetSysParam(PN_Date(),inttodate(endT));
        stocks := getAbkbydate("A股",endT);
        a := array();
        
        for i:=0 to length(stocks)-1 do
        begin
            setsysparam(pn_stock(),stocks[i]); 
            a[i]['StockID'] := stocks[i];
            a[i]['PctChg'] := StockZf3();
            a[i]['IsST'] := IsST_3(endT);
            a[i]['FloatShares'] := StockNegotiableShares(endT);                    
        end;
        
        return a;
    """ % (dateInt);

    # 天软去除的数据格式为list[dict[field, value]]，fieldbyte形式的，其他是数值。
    data2 = ts.RemoteExecute(tsCode2, {});

    if data2[0] == 0:
        stocks = [str(s[b'StockID'], encoding='utf-8') for s in data2[1]];
        pctChgLi = [s[b'PctChg'] for s in data2[1]];
        stLi = [s[b'IsST'] for s in data2[1]];
        floatSharesLi = [s[b'FloatShares'] for s in data2[1]];

        supplementDf = DataFrame({"PctChg":pctChgLi, "IsST":stLi, "FloatShares":floatSharesLi}, index = stocks);
        if klineDf is not None:
            klineDf['PctChg'] = supplementDf['PctChg'];
            klineDf['IsST'] = supplementDf['IsST'];
            klineDf['TurnOver'] = klineDf['Volume'] / supplementDf['FloatShares'] * 100;



    if connectionSafe:
        logoff();

    return klineDf;


# 获取指定日期的股票1分钟K线
def getStock1MinKLines(dateInt, stockLi=None, connectionSafe = True):
    login();
    if stockLi is None:
        stockLi = getListedStock(dateInt, returnType=str, connectionSafe=False);

    stockCodeArr = sqlCF.mergeStr(stockLi);


    ## 获取基本的K线字段
    tsCode = """
        setsysparam(pn_cycle(), cy_1m());
        a:=array(%s);
        t:=select ['StockID'],DateTimeToStr(['date']) as 'time',['open'],['high'],['low'],['close'], ['vol'], ['amount'], ['yclose'] as 'AdjPreClose',
            (['close']/['yclose'] - 1) * 100 as 'PctChg'
            from markettable datekey %dT to %dT+0.999 of a end;
        return t;
    """ % (stockCodeArr, dateInt, dateInt);

    #print("get data");
    # 天软去除的数据格式为list[dict[field, value]]，fieldbyte形式的，其他是数值。
    data = ts.RemoteExecute(tsCode, {});

    #print("process data");
    # data[0] == 0 表示返回没有错误。
    if data[0] == 0:
        stocks = [str(s[b'StockID'], encoding='utf-8') for s in data[1]];
        timeLi = [datetime.strptime(str(s[b'time'], encoding='utf-8'),"%Y-%m-%d %H:%M:%S") for s in data[1]];
        openLi = [s[b'open'] for s in data[1]];
        highLi = [s[b'high'] for s in data[1]];
        lowLi = [s[b'low'] for s in data[1]];
        closeLi = [s[b'close'] for s in data[1]];
        volLi = [s[b'vol'] for s in data[1]];
        amtLi = [s[b'amount'] for s in data[1]];
        preClsLi = [s[b'AdjPreClose'] for s in data[1]];
        pctChgLi = [s[b'PctChg'] for s in data[1]];

        klineDf = DataFrame({"Stock": stocks, "DateTime": timeLi, "Open":openLi, "High":highLi, "Low":lowLi, "Close":closeLi, "Volume":volLi,
                             "Amount":amtLi, "AdjPreClose":preClsLi, "PctChg":pctChgLi});
    else:
        klineDf = None;

    if connectionSafe:
        logoff();
    #print("return data");
    return klineDf;

# stock 为天软格式，如SH600000, SZ000001;
# connectionSafe: 为True时，会在结束时断开连接
def getSingleStock1MinKLines(stock:str, dateInt: int, connectionSafe = True):
    login();

    tsCode = """
        SetSysParam(pn_stock(), "%s"); // 设置当前股票
        SetSysParam(pn_cycle(), cy_1m());
        SetSysParam(pn_date(), inttodate(%d) + 15 / 24); // 14 / 24就是15点钟。
        N := 240; // 取240日
        Return Nday(N, "time", DateTimeToStr(sp_time()), "Open", open(), "High", high(), "Low", low(), "Close", close(),
         "Volume", vol(), "Amount", amount(), "AdjPreClose", Sys_PrevClose(), "PctChg", (close() / Sys_PrevClose() - 1) * 100);
    """ % (stock, dateInt);

    # 天软去除的数据格式为list[dict[field, value]]，fieldbyte形式的，其他是数值。
    data = ts.RemoteExecute(tsCode, {});

    # data[0] == 0 表示返回没有错误。
    if data[0] == 0:
        timeLi = [datetime.strptime(str(s[b'time'], encoding='utf-8'),"%Y-%m-%d %H:%M:%S") for s in data[1]];
        openLi = [s[b'Open'] for s in data[1]];
        highLi = [s[b'High'] for s in data[1]];
        lowLi = [s[b'Low'] for s in data[1]];
        closeLi = [s[b'Close'] for s in data[1]];
        volLi = [s[b'Volume'] for s in data[1]];
        amtLi = [s[b'Amount'] for s in data[1]];
        adjPreClsLi = [s[b'AdjPreClose'] for s in data[1]];
        pctChgLi = [s[b'PctChg'] for s in data[1]];

        klineDf = DataFrame({"Stock": stock, "Time": timeLi, "Open":openLi, "High":highLi, "Low":lowLi, "Close":closeLi, "Volume":volLi,
                             "Amount":amtLi, "AdjPreClose":adjPreClsLi, "PctChg":pctChgLi}, );

        # 万一有当天不足240根K线的股票，可能会导入非当天的K线，需要检查下日期。
        klineDf = klineDf.loc[klineDf["Time"] > datetime.strptime(str(dateInt), "%Y%m%d"),:];
    else:
        klineDf = None;

    if connectionSafe:
        logoff();

    return klineDf;


# 获取当天ST的股票
def getSTStocks(dateInt, returnType=str, connectionSafe=True):
    login();

    tsCode = """
        endT := %d;
        stocks := getAbkbydate("A股",endT);
        data := array();
        n := 0;
        for i:=0 to length(stocks)-1 do
        begin
           setsysparam(pn_stock(),stocks[i]);
           if IsST_3(endt)=0 then continue;
           data[n]:=stocks[i];
           n++;
        end;
        return data;
    """ % (dateInt);

    data = ts.RemoteExecute(tsCode, {});
    if data[0] == 0:
        if returnType == str:
            stockLi = [str(s, encoding='utf-8') for s in data[1]];
        elif returnType == int:
            stockLi = [int(str(s, encoding='utf-8')[2:]) for s in data[1]];
        else:
            raise Exception("getBK暂时不支持返回此格式");
    else:
        stockLi = None;

    if connectionSafe:
        logoff();

    return stockLi;

