#!/usr/bin/env python
# coding: utf-8

# In[34]:


import pandas as pd
import numpy as np
import os
from datetime import datetime
def to_last_season(datestamp):
    '''将当前季末日期转换为上一季末日期'''
    last_season = (pd.Series(datestamp) - pd.DateOffset(months=3)).apply(
                    lambda x: x + pd.to_timedelta('1 days') if x.month == 3 else x)
    return last_season

def intdate(datestamp):
    '''日期转换为8位整数'''
    return int(datestamp.strftime('%Y%m%d'))

def non_rec_deduct_profit_single_growth(df):
    profit = df['扣非净利润'].values
    ss1 = pd.to_datetime(df['截止日'], format='%Y%m%d') # 当季
    ss2 = to_last_season(ss1) # 上一季
    ss3 = ss1 - pd.DateOffset(years=1) # 上一年同季
    ss4 = to_last_season(ss3) # 上一年上一季
    profit_mat = [] # [当季利润,上季利润,去年利润,去年上季利润,单季利润,去年单季利润]
    single_profit_pctchg = [] # 单季度利润同比
    for i in range(ss1.shape[0]):
        s1 = ss1[:i+1].max() # 当前公告日能获取的最新报告期
        s2 = to_last_season(s1)[0]
        s3 = s1 - pd.DateOffset(years=1)
        s4 = to_last_season(s3)[0]
        latest_seasons = [intdate(s1), intdate(s2), intdate(s3), intdate(s4)] #计算最新同比需要的报告期
        adjust_season = np.where(np.in1d(latest_seasons, intdate(ss1[i])))[0]
        
        if len(adjust_season):
            adjust_season = adjust_season[0]
            if adjust_season == 0:
                # 最新报告期，计算最新同比
                if ss2[i].month != 12:
                    s2 = np.where(ss1[:i] == ss2[i])[0]
                    s4 = np.where(ss1[:i] == ss4[i])[0]
                    p2 = profit[s2[-1]] if len(s2) else np.nan
                    p4 = profit[s4[-1]] if len(s4) else np.nan
                else:
                    p2 = 0
                    p4 = 0
                s3 = np.where(ss1[:i] == ss3[i])[0]
                p3 = profit[s3[-1]] if len(s3) else np.nan
                p1 = profit[i]
                profit_mat.append([p1, p2, p3, p4, p1-p2, p3-p4])
            else:
                # 非最新报告期，但调整数据在最新报告期计算的范围内
                profit_adjust = profit_mat[-1].copy()
                profit_adjust[adjust_season] = 0 if                      latest_seasons[adjust_season] % 10000 == 1231 and                     adjust_season in [1, 3] else profit[i]# 调整相应季度为最新数据
                if adjust_season <= 1:
                    profit_adjust[4] = profit_adjust[0] - profit_adjust[1]
                else:
                    profit_adjust[5] = profit_adjust[2] - profit_adjust[3]
                profit_mat.append(profit_adjust)
        else:
            # 非最新报告期，但调整数据不在最新报告期的计算范围内
            profit_mat.append(profit_mat[-1])
            
        single_profit_pctchg.append((profit_mat[-1][4] - profit_mat[-1][5]) / abs(profit_mat[-1][5])
                                    if profit_mat[-1][5] != 0 else np.nan)
    profit_mat = pd.DataFrame(profit_mat, columns=['当季', 
                                                   '上季', 
                                                   '去年当季', 
                                                   '去年上季', 
                                                   '单季',
                                                   '去年单季'])
    single_profit_pctchg = pd.Series(single_profit_pctchg, name='同比')
        
    return pd.concat([pd.DataFrame({'StockID': df['StockID'],
                                    '截止日': ss1.apply(intdate),
                                    '公布日': df['公布日']}),
                      profit_mat,
                      single_profit_pctchg,
                      df['公告类型']], axis=1)

# stocklist = os.listdir('D:/数据/天软基本面数据/42.主要财务指标')
# df_list = []
# for stock in stocklist:
#     df = pd.read_csv('D:/数据/天软基本面数据/42.主要财务指标/' + stock)
#     factor = non_rec_deduct_profit_single_growth(df)
#     df_list.append(factor)
# factor_df = pd.concat(df_list, axis=1, join='outer')
# factor_df.fillna(method='ffill', inplace=True)
#     factor_df = factor_df.join(factor, how='outer')


# In[35]:


# 业绩预测
# data = pd.read_csv('D:/数据/天软基本面数据/40.业绩预测/SZ000048.csv')
# df1 = pd.DataFrame({'StockID': data['StockID'],
#                    '截止日': data['截止日'],
#                    '公布日': data['公布日'],
#                    '归母净利润': (data['盈利金额上限'] + data['盈利金额下限']) * 10000/2,
#                    '公告类型': '业绩预测'})
#  业绩快报
# data = pd.read_csv('D:/数据/天软基本面数据/41.业绩快报/SZ000048.csv')
# df2 = pd.DataFrame({'StockID': data['StockID'],
#                    '截止日': data['截止日'],
#                    '公布日': data['公布日'],
#                    '归母净利润': data['归属于母公司所有者净利润'],
#                    '公告类型': '业绩快报'})
# 正式报告
filelist = os.listdir('D:/数据/天软基本面数据/46.合并利润分配表')
for file in filelist:
    data1 = pd.read_csv('D:/数据/天软基本面数据/42.主要财务指标/' + file)
    data2 = pd.read_csv('D:/数据/天软基本面数据/46.合并利润分配表/' + file)
    A = pd.DataFrame({'StockID': data1['StockID'],
                      '截止日': data1['截止日'],
                      '公布日': data1['公布日'],
                     '首次公布扣非净利润': data1['扣除非经常性损益后的净利润']})
    B = pd.DataFrame({'StockID': data2['StockID'],
                       '截止日': data2['截止日'],
                       '公布日': data2['公布日'],
                       '归母净利润': data2['归属于母公司所有者净利润']})
    A.index = A['截止日'].astype('str') + A['公布日'].astype('str')
    B.index = B['截止日'].astype('str') + B['公布日'].astype('str')
    df3 = B.join(A['首次公布扣非净利润'], how='left')
    df3.sort_values(['公布日', '截止日'], inplace=True)
    df3.reset_index(drop=True, inplace=True)
    df3['非经常性损益'] = df3['归母净利润'] - df3['首次公布扣非净利润']
    df3['非经常性损益'] = df3.groupby('截止日')['非经常性损益'].transform(lambda x: x.fillna(method='ffill'))
    df3['扣非净利润'] = df3['归母净利润'] - df3['非经常性损益']
    df3 = df3[['StockID', '截止日', '公布日', '扣非净利润']]
    df3['公告类型'] = '正式报告'

#     df = pd.concat([df1, df2, df3], axis=0)
#     df.sort_values(['公布日', '截止日'], inplace=True)
#     df.reset_index(inplace=True)
    result = non_rec_deduct_profit_single_growth(df3)
    result.to_csv('D:/数据/天软基本面数据/单季度扣非同比/' + file, index=False)

